/*
 * circlepi+.c
 *
 * Computes an approximation of Pi through a disk area
 *
 * Compile with: gcc -Ofast -Wall circlepi+.c -o circlepi+
 * Execute with: ./circlepi+ 1000
 *
 * Author: Xavier Bourgeois - xavier@monorailc.at
 * License: Creative Commons - BY-NC
 * Article: https://monorailc.at/cms/2025-05-20-comparaison-de-langages-de-programmation--5----autre-algorithme.html
 * Date: 2025-05-20
 */

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <math.h>

unsigned long compute_int32(int n) {
	unsigned long count = 0;
	unsigned int hcount = 0;
	int i, j;
	int inf = floor(.7*n);
	int sup = ceil(.71*n);

	count = n;		// 0

	for(i = inf+1; i <= sup; i++) {			// area 1
		for(j = inf+1; j <= sup; j++) {
			if(i*i + j*j < n*n)
				count++;
			if(i*i + j*j == n*n)
				hcount++;
		}
	}

	count += inf*inf;						// area 2

	for(i = inf+1; i <= sup; i++) {			// area 3
		for(j = 1; j <= inf; j++) {
			if(i*i + j*j < n*n)
				count+=2;
			if(i*i + j*j == n*n)
				hcount+=2;
		}
	}
	for(i = sup+1; i <= n; i++) {			// area 4
		for(j = 1; j <= inf; j++) {
			if(i*i + j*j < n*n)
				count+=2;
			if(i*i + j*j == n*n)
				hcount+=2;
		}
	}
	for(i = sup+1; i <= n; i++) {			// area 5
		for(j = inf+1; j <= sup; j++) {
			if(i*i + j*j < n*n)
				count+=2;
			if(i*i + j*j == n*n)
				hcount+=2;
		}
	}

	/*
	for(i = 0; i <= n; i++) {
		for(j = 1; j <= n; j++) {
			if(i*i + j*j < n*n) {
				count++;
			}
			if(i*i + j*j == n*n) {
				hcount++;
			}
		}
	}*/

	printf("%ld, %d, %f\n", count, n*n, 4.0 * (double) count/(n*n));
	return count+hcount/2;
}

unsigned long compute_int64(long int n) {
	unsigned long count = 0;
	unsigned int hcount = 0;
	long int i, j; // computation with long ints is 3 times slower
	long int inf = floor(.7*n);
	long int sup = ceil(.71*n);

	count = n;		// 0

	for(i = inf+1; i <= sup; i++) {			// area 1
		for(j = inf+1; j <= sup; j++) {
			if(i*i + j*j < n*n)
				count++;
			if(i*i + j*j == n*n)
				hcount++;
		}
	}

	count += inf*inf;						// area 2

	for(i = inf+1; i <= sup; i++) {			// area 3
		for(j = 1; j <= inf; j++) {
			if(i*i + j*j < n*n)
				count+=2;
			if(i*i + j*j == n*n)
				hcount+=2;
		}
	}
	for(i = sup+1; i <= n; i++) {			// area 4
		for(j = 1; j <= inf; j++) {
			if(i*i + j*j < n*n)
				count+=2;
			if(i*i + j*j == n*n)
				hcount+=2;
		}
	}
	for(i = sup+1; i <= n; i++) {			// area 5
		for(j = inf+1; j <= sup; j++) {
			if(i*i + j*j < n*n)
				count+=2;
			if(i*i + j*j == n*n)
				hcount+=2;
		}
	}

	return count+hcount/2;
}

int main(int argc, char **argv) {
	unsigned long count;
	long n;

	if(argc < 1)
		return 1;
	n = (int) abs(atoi(argv[1]));
	if(2*n*n > INT_MAX) {	// 2*n*n >= UINT_MAX -> use long ints to avoid overflows, triggers at 32767
		count = compute_int64(n);
	}
	else
		count = compute_int32((int) n);

	//printf("%.16lf\n", (double) count / (n*n));
	printf("%ld\t%.16lf\t%.16lf\n", n, 4.0 * (double) count / (n*n), M_PI - 4.0 * (double) count / (n*n));
	return 0;
}
