/*
 * circlepi_omp.cpp
 *
 * Computes an approximation of Pi through a disk area
 *
 * Compile with: g++ -fopenmp -Ofast -Wall -std=c++20 circlepi_omp.cpp -o circlepi_omp
 * Execute with: ./circlepi_omp 1000
 *
 * Author: Xavier Bourgeois - xavier@monorailc.at
 * License: Creative Commons - BY-NC
 * Article: https://monorailc.at/cms/2025-05-20-comparaison-de-langages-de-programmation--5----autre-algorithme.html
 * Date: 2025-05-17
 */

#include <iostream>
#include <cmath>
#include <cstdio>
#include <climits>
#include <vector>
#include <omp.h>

//#define ABS(x) x>0?x:-x // unused, no major difference

unsigned long compute_int(auto n) {
	unsigned long count = 0;

	#if defined(_OPENMP)
	int num_threads = omp_get_max_threads();
	#else
	int num_threads = 1;
	#endif
	//std::cout << "Number of threads: " << num_threads << std::endl;
	std::vector<unsigned long>local_counts(num_threads, 0);

	#pragma omp parallel
	{
		#if defined(_OPENMP)
		int thread_id = omp_get_thread_num();
		#else
		int thread_id = 0;
		#endif

		unsigned long local_count = 0;
		#pragma omp for
		for(auto i = -n; i <= n; i++) {
			for(auto j = -n; j <= n; j++) {
				if(i*i + j*j < n*n) {
					local_count++;
				}
			}
		}
		local_counts[thread_id] = local_count;
	}
	for (int i = 0; i < num_threads; ++i) {
		count += local_counts[i];
	}

	return count;
}

int main(int argc, char **argv) {
	unsigned long count;

	if(argc < 1)
		return 1;
	long n = abs(atoi(argv[1]));
	if(2*n*n < INT_MAX)
		count = compute_int((int)n);
	else
		count = compute_int((long)n);
	std::cout<<n<<'\t'<<(double) count / (n*n)<<'\t'<<M_PI - (double) count / (n*n)<<'\n';
	return 0;
}
